@@ -1,10 +1,10 @@
 /* Class.java -- Representation of a Java class.
-   Copyright (C) 1998, 1999, 2000, 2002, 2003, 2004
+   Copyright (C) 1998, 1999, 2000, 2002, 2003, 2004, 2005
    Free Software Foundation
 
 Modifications Copyright (C) 2004 by Etienne Gagnon.
 Modifications Copyright (C) 2004 by David Belanger.
-Modifications Copyright (C) 2004 by Grzegorz prokopski.
+Modifications Copyright (C) 2004, 2005 by Grzegorz Prokopski.
 Modifications Copyright (C) 2004 by Laurent Martelli.
 
 This file is part of GNU Classpath.
@@ -43,6 +43,8 @@ exception statement from your version. */
 
 package java.lang;
 
+import gnu.classpath.VMStackWalker;
+
 import java.io.InputStream;
 import java.io.Serializable;
 import java.lang.reflect.Constructor;
@@ -250,15 +252,11 @@ public final class Class implements Serializable
    */
   public static Class forName(String name) throws ClassNotFoundException
   {
-    /*
     Class result = VMClass.forName (name);
     if (result == null)
       result = Class.forName(name, true,
-			     VMSecurityManager.getClassContext()[1].getClassLoader());
+	VMStackWalker.getCallingClassLoader());
     return result;
-    */
-    return forName(name, true, VMSecurityManager.currentClassLoader());
-    
   }
 
   /**
@@ -363,9 +361,8 @@ public final class Class implements Serializable
         SecurityManager sm = SecurityManager.current;
         if (sm != null)
           {
-            // Get the calling class and classloader
-            Class c = VMSecurityManager.getClassContext()[1];
-            ClassLoader cl = c.getClassLoader();
+            // Get the calling classloader
+            ClassLoader cl = VMStackWalker.getCallingClassLoader();
             if (cl != null)
               sm.checkPermission(new RuntimePermission("getClassLoader"));
           }
@@ -444,9 +441,8 @@ public final class Class implements Serializable
     SecurityManager sm = SecurityManager.current;
     if (sm != null)
       {
-        // Get the calling class and classloader
-        Class c = VMSecurityManager.getClassContext()[1];
-        ClassLoader cl = VMClass.getClassLoader(c);
+        // Get the calling classloader
+	ClassLoader cl = VMStackWalker.getCallingClassLoader();
         if (cl != null && !cl.isAncestorOf(loader))
           sm.checkPermission(new RuntimePermission("getClassLoader"));
       }
@@ -1471,11 +1467,14 @@ public final class Class implements Serializable
 	  }
 	if (constructor == null)
 	  throw new InstantiationException(getName());
-	if (!Modifier.isPublic(constructor.getModifiers()))
+	if (!Modifier.isPublic(constructor.getModifiers())
+            || !Modifier.isPublic(getModifiers()))
 	  {
 	    final Constructor finalConstructor = constructor;
-	    AccessController.doPrivileged(new PrivilegedAction() {
-		public Object run() {
+	    AccessController.doPrivileged(new PrivilegedAction()
+	      {
+		public Object run()
+	        {
 		  finalConstructor.setAccessible(true);
 		  return null;
 		}
@@ -1488,10 +1487,12 @@ public final class Class implements Serializable
 	  }	    
       }
     int modifiers = constructor.getModifiers();
-    if (!Modifier.isPublic(modifiers))
+    if (!Modifier.isPublic(modifiers)
+        || !Modifier.isPublic(getModifiers()))
       {
-	Class caller = VMSecurityManager.getClassContext()[1];
-	if (caller != this &&
+	Class caller = VMStackWalker.getCallingClass();
+	if (caller != null &&
+	    caller != this &&
 	    (Modifier.isPrivate(modifiers)
 	     || getClassLoader() != caller.getClassLoader()
 	     || !getPackagePortion(getName())
