@@ -1,9 +1,9 @@
 /* ClassLoader.java -- responsible for loading classes into the VM
-   Copyright (C) 1998, 1999, 2001, 2002, 2003, 2004 Free Software Foundation, Inc.
+   Copyright (C) 1998, 1999, 2001, 2002, 2003, 2004, 2005 Free Software Foundation, Inc.
 
 Modifications Copyright (C) 2004 by Etienne Gagnon.
 Modifications Copyright (C) 2004 by David Belanger.
-Modifications Copyright (C) 2004 by Grzegorz prokopski.
+Modifications Copyright (C) 2004, 2005 by Grzegorz Prokopski.
 
 This file is part of GNU Classpath.
 
@@ -43,6 +43,7 @@ exception statement from your version. */
 package java.lang;
 
 import gnu.classpath.SystemProperties;
+import gnu.classpath.VMStackWalker;
 import gnu.java.util.DoubleEnumeration;
 import gnu.java.util.EmptyEnumeration;
 
@@ -121,7 +122,7 @@ import java.util.StringTokenizer;
  *
  * @author John Keiser
  * @author Mark Wielaard
- * @author Eric Blake <ebb9@email.byu.edu>
+ * @author Eric Blake (ebb9@email.byu.edu)
  * @see Class
  * @since 1.0
  * @status still missing 1.4 functionality
@@ -549,7 +550,7 @@ public abstract class ClassLoader
    *   {
    *     String packageName = name.substring(0, lastDot);
    *     // Look if the package already exists
-   *     if (getPackage(pkg) == null)
+   *     if (getPackage(packageName) == null)
    *       {
    *         // define the package
    *         definePackage(packageName, ...);
@@ -727,8 +728,7 @@ public abstract class ClassLoader
     SecurityManager sm = SecurityManager.current;
     if (sm != null)
       {
-        Class c = VMSecurityManager.getClassContext()[1];
-        ClassLoader cl = c.getClassLoader();
+	ClassLoader cl = VMStackWalker.getCallingClassLoader();
 	if (cl != null && ! cl.isAncestorOf(this))
           sm.checkPermission(new RuntimePermission("getClassLoader"));
       }
@@ -973,8 +973,7 @@ public abstract class ClassLoader
     SecurityManager sm = SecurityManager.current;
     if (sm != null)
       {
-	Class c = VMSecurityManager.getClassContext()[1];
-	ClassLoader cl = c.getClassLoader();
+	ClassLoader cl = VMStackWalker.getCallingClassLoader();
 	if (cl != null && cl != StaticData.systemClassLoader)
 	  sm.checkPermission(new RuntimePermission("getClassLoader"));
       }
@@ -1207,10 +1206,9 @@ public abstract class ClassLoader
 	  {
 	    File f = new File(tok.nextToken());
 	    File[] files = f.listFiles();
-	    for (int i = 0; i < files.length; i++)
-	      {
+	    if (files != null)
+	      for (int i = 0; i < files.length; i++)
 		list.add(files[i].toURL());
-	      }
 	  }
 	catch(Exception x)
 	  {
@@ -1263,8 +1261,13 @@ public abstract class ClassLoader
 
   static ClassLoader defaultGetSystemClassLoader()
   {
-    ClassLoader extClassLoader =
-	new URLClassLoader(getExtClassLoaderUrls(), null);
+    URL[] extURLs = getExtClassLoaderUrls();
+    ClassLoader extClassLoader;
+    if (extURLs.length > 0)
+      extClassLoader = new URLClassLoader(getExtClassLoaderUrls(), null);
+    else
+      extClassLoader = null;
+
     ClassLoader systemClassLoader =
 	new URLClassLoader(getSystemClassLoaderUrls(), extClassLoader)
 	{
