@@ -1,9 +1,9 @@
 /* Runtime.java -- access to the VM process
-   Copyright (C) 1998, 2002, 2003, 2004 Free Software Foundation
+   Copyright (C) 1998, 2002, 2003, 2004, 2005 Free Software Foundation
 
 Modifications Copyright (C) 2004 by Etienne Gagnon.
 Modifications Copyright (C) 2004 by David Belanger.
-Modifications Copyright (C) 2004 by Grzegorz prokopski.
+Modifications Copyright (C) 2004, 2005 by Grzegorz Prokopski.
 
 This file is part of GNU Classpath.
 
@@ -43,6 +43,7 @@ exception statement from your version. */
 package java.lang;
 
 import gnu.classpath.SystemProperties;
+import gnu.classpath.VMStackWalker;
 
 import java.io.File;
 import java.io.IOException;
@@ -50,7 +51,6 @@ import java.io.InputStream;
 import java.io.OutputStream;
 import java.util.HashSet;
 import java.util.Iterator;
-import java.util.Properties;
 import java.util.Set;
 import java.util.StringTokenizer;
 
@@ -58,7 +58,7 @@ import java.util.StringTokenizer;
  * Runtime represents the Virtual Machine.
  *
  * @author John Keiser
- * @author Eric Blake <ebb9@email.byu.edu>
+ * @author Eric Blake (ebb9@email.byu.edu)
  * @author Jeroen Frijters
  */
 // No idea why this class isn't final, since you can't build a subclass!
@@ -97,18 +97,41 @@ public class Runtime
   {
     if (current != null)
       throw new InternalError("Attempt to recreate Runtime");
+    
+    // If used by underlying VM this contains the directories where Classpath's own
+    // native libraries are located.
+    String bootPath = SystemProperties.getProperty("gnu.classpath.boot.library.path", "");
+    
+    // If properly set by the user this contains the directories where the application's
+    // native libraries are located. On operating systems where a LD_LIBRARY_PATH environment
+    // variable is available a VM should preset java.library.path with value of this
+    // variable.
     String path = SystemProperties.getProperty("java.library.path", ".");
     String pathSep = SystemProperties.getProperty("path.separator", ":");
     String fileSep = SystemProperties.getProperty("file.separator", "/");
-    StringTokenizer t = new StringTokenizer(path, pathSep);
-    libpath = new String[t.countTokens()];
-    for (int i = 0; i < libpath.length; i++)
-      {
-        String prefix = t.nextToken();
-        if (! prefix.endsWith(fileSep))
-          prefix += fileSep;
-        libpath[i] = prefix;
-      }
+
+    StringTokenizer t1 = new StringTokenizer(bootPath, pathSep);
+    StringTokenizer t2 = new StringTokenizer(path, pathSep);
+    libpath = new String[t1.countTokens() + t2.countTokens()];
+    
+    int i = 0;
+    while(t1.hasMoreTokens()) {
+      String prefix = t1.nextToken();
+      if (! prefix.endsWith(fileSep))
+        prefix += fileSep;
+      
+      libpath[i] = prefix;
+      i++;
+    }
+    
+    while(t2.hasMoreTokens()) {
+      String prefix = t2.nextToken();
+      if (! prefix.endsWith(fileSep))
+        prefix += fileSep;
+  
+      libpath[i] = prefix;
+      i++;
+    }
   }
 
   /**
@@ -633,16 +656,33 @@ public class Runtime
    * before the final ".so" if the VM was invoked by the name "java_g". There
    * may be a security check, of <code>checkLink</code>.
    *
+   * <p>
+   * The library is loaded using the class loader associated with the
+   * class associated with the invoking method.
+   *
    * @param filename the file to load
    * @throws SecurityException if permission is denied
    * @throws UnsatisfiedLinkError if the library is not found
    */
   public void load(String filename)
+  {
+    load(filename, VMStackWalker.getCallingClassLoader());
+  }
+
+  /**
+   * Same as <code>load(String)</code> but using the given loader.
+   *
+   * @param filename the file to load
+   * @param loader class loader, or <code>null</code> for the boot loader
+   * @throws SecurityException if permission is denied
+   * @throws UnsatisfiedLinkError if the library is not found
+   */
+  void load(String filename, ClassLoader loader)
   {
     SecurityManager sm = SecurityManager.current; // Be thread-safe!
     if (sm != null)
       sm.checkLink(filename);
-    if (loadLib(filename) == 0)
+    if (loadLib(filename, loader) == 0)
       throw new UnsatisfiedLinkError("Could not load library " + filename);
   }
 
@@ -650,15 +690,16 @@ public class Runtime
    * Do a security check on the filename and then load the native library.
    *
    * @param filename the file to load
+   * @param loader class loader, or <code>null</code> for the boot loader
    * @return 0 on failure, nonzero on success
    * @throws SecurityException if file read permission is denied
    */
-  private static int loadLib(String filename)
+  private static int loadLib(String filename, ClassLoader loader)
   {
     SecurityManager sm = SecurityManager.current; // Be thread-safe!
     if (sm != null)
       sm.checkRead(filename);
-    return VMRuntime.nativeLoad(filename);
+    return VMRuntime.nativeLoad(filename, loader);
   }
 
   /**
@@ -673,6 +714,17 @@ public class Runtime
    * <code>System.mapLibraryName(libname)</code>. There may be a security
    * check, of <code>checkLink</code>.
    *
+   * <p>Note: Besides <code>java.library.path</code> a VM may chose to search
+   * for native libraries in a path that is specified by the
+   * <code>gnu.classpath.boot.library.path</code> system property. However
+   * this is for internal usage or development of GNU Classpath only.
+   * <b>A Java application must not load a non-system library by changing
+   * this property otherwise it will break compatibility.</b></p>
+   *
+   * <p>
+   * The library is loaded using the class loader associated with the
+   * class associated with the invoking method.
+   *
    * @param libname the library to load
    *
    * @throws SecurityException if permission is denied
@@ -682,55 +734,49 @@ public class Runtime
    * @see ClassLoader#findLibrary(String)
    */
   public void loadLibrary(String libname)
+  {
+    loadLibrary(libname, VMStackWalker.getCallingClassLoader());
+  }
+
+  /**
+   * Same as <code>loadLibrary(String)</code> but using the given loader.
+   *
+   * @param libname the library to load
+   * @param loader class loader, or <code>null</code> for the boot loader
+   * @throws SecurityException if permission is denied
+   * @throws UnsatisfiedLinkError if the library is not found
+   */
+  void loadLibrary(String libname, ClassLoader loader)
   {
     SecurityManager sm = SecurityManager.current; // Be thread-safe!
     if (sm != null)
       sm.checkLink(libname);
-
     String filename;
-    ClassLoader cl = VMSecurityManager.currentClassLoader();
-    if (cl != null)
+    if (loader != null && (filename = loader.findLibrary(libname)) != null)
       {
-        filename = cl.findLibrary(libname);
-        if (filename != null)
-          {
-            if (loadLib(filename) != 0)
-	      return;
-	    else
-	      throw new UnsatisfiedLinkError("Could not load library " + filename);
-          }
+	if (loadLib(filename, loader) != 0)
+	  return;
       }
-
-    filename = VMRuntime.mapLibraryName(libname);
-    for (int i = 0; i < libpath.length; i++)
-      if (loadLib(libpath[i] + filename) != 0)
-	return;
-
-    
-    // Temporary commented out until problem with bootclass libraries
-    // is resolved.
-    //
-    // throw new UnsatisfiedLinkError("Could not find library " + libname + ".");
-
-    /* bootstrap class loader */
-
-    /* Note to Classpath developers: Be careful not to confuse
-     * "bootstrap class loader" and "system class loader"!  They are
-     * distinct beasts.
-     */
-    {
-      String path = SystemProperties.getProperty("sablevm.boot.library.path");
-//      String path = VMRuntime.getBootstrapLibraryPath();
-      
-      if (!path.endsWith ("/"))
-	{
-	  path += "/";
-	}
-      
-      String fullName = path + System.mapLibraryName(libname);
-      
-      load (fullName);
-    }
+    else
+      {
+	filename = VMRuntime.mapLibraryName(libname);
+	for (int i = 0; i < libpath.length; i++)
+	  if (loadLib(libpath[i] + filename, loader) != 0)
+	    return;
+      /* finally try the boot library path */
+	String path = SystemProperties.getProperty("sablevm.boot.library.path");
+	if (!path.endsWith ("/"))
+	  {
+	    path += "/";
+	  }
+	try {
+	  load (path + filename);
+	  return;
+	  /* we catch this exception to provide a nicer error message */
+	} catch (UnsatisfiedLinkError e) {}
+      }
+    throw new UnsatisfiedLinkError("Native library `" + libname
+      + "' not found (as file `" + filename + "') in gnu.classpath.boot.library.path and java.library.path");
   }
 
   /**
