@@ -424,6 +424,18 @@ public final class String implements Serializable, Comparable, CharSequence
       }
   }
 
+  /**
+   * Creates a new String using the character sequence represented by
+   * the StringBuilder. Subsequent changes to buf do not affect the String.
+   *
+   * @param buffer StringBuilder to copy
+   * @throws NullPointerException if buffer is null
+   */
+  public String(StringBuilder buffer)
+  {
+    this(buffer.value, 0, buffer.count);
+  }
+
   /**
    * Special constructor which can share an array when safe to do so.
    *
