@@ -4,7 +4,7 @@
 
 Modifications Copyright (C) 2004 by Etienne Gagnon.
 Modifications Copyright (C) 2004 by David Belanger.
-Modifications Copyright (C) 2004 by Grzegorz prokopski.
+Modifications Copyright (C) 2004, 2005 by Grzegorz Prokopski.
 
 This file is part of GNU Classpath.
 
@@ -42,6 +42,8 @@ exception statement from your version. */
 
 package java.lang;
 
+import java.util.Map;
+import java.util.WeakHashMap;
 
 /* Written using "Java Class Libraries", 2nd edition, ISBN 0-201-31002-3
  * "The Java Language Specification", ISBN 0-201-63451-1
@@ -135,6 +137,11 @@ public class Thread implements Runnable
   /** The next thread number to use. */
   private static int numAnonymousThreadsCreated;
 
+  /** Thread local storage. Package accessible for use by
+    * InheritableThreadLocal.
+    */
+  WeakHashMap locals;
+
   /**
    * Allocates a new <code>Thread</code> object. This constructor has
    * the same effect as <code>Thread(null, null,</code>
@@ -372,8 +379,6 @@ public class Thread implements Runnable
 
   /* SableVM specific code */
   byte[] vmData;
-  Object sleeper = new Object();
-
 
   Thread(String name, ThreadGroup group)
   {
@@ -381,6 +386,7 @@ public class Thread implements Runnable
     this.name = name;
     this.runnable = null;
     this.priority = MAX_PRIORITY;
+    this.vmThread = new VMThread(this);
     
     group.addThread(this);
   }
@@ -687,9 +693,7 @@ public class Thread implements Runnable
   public final synchronized void resume()
   {
     checkAccess();
-    VMThread t = vmThread;
-    if (t != null)
-	t.resume();
+    VMThread.resume(vmData);
   }
   
   /**
@@ -808,11 +812,11 @@ public class Thread implements Runnable
    * are no guarantees which thread will be next to run, but most VMs will
    * choose the highest priority thread that has been waiting longest.
    *
-   * @param ms the number of milliseconds to sleep
-   * @throws InterruptedException if the Thread is interrupted; it's
-   *         <i>interrupted status</i> will be cleared
-   * @see #notify()
-   * @see #wait(long)
+   * @param ms the number of milliseconds to sleep.
+   * @throws InterruptedException if the Thread is (or was) interrupted;
+   *         it's <i>interrupted status</i> will be cleared
+   * @throws IllegalArgumentException if ms is negative
+   * @see #interrupt()
    */
   public static void sleep(long ms) throws InterruptedException
   {
@@ -824,26 +828,30 @@ public class Thread implements Runnable
    * time. The Thread will not lose any locks it has during this time. There
    * are no guarantees which thread will be next to run, but most VMs will
    * choose the highest priority thread that has been waiting longest.
-   *
-   * <p>Note that 1,000,000 nanoseconds == 1 millisecond, but most VMs do
-   * not offer that fine a grain of timing resolution. Besides, there is
-   * no guarantee that this thread can start up immediately when time expires,
-   * because some other thread may be active.  So don't expect real-time
-   * performance.
+   * <p>
+   * Note that 1,000,000 nanoseconds == 1 millisecond, but most VMs
+   * do not offer that fine a grain of timing resolution. When ms is
+   * zero and ns is non-zero the Thread will sleep for at least one
+   * milli second. There is no guarantee that this thread can start up
+   * immediately when time expires, because some other thread may be
+   * active.  So don't expect real-time performance.
    *
    * @param ms the number of milliseconds to sleep
    * @param ns the number of extra nanoseconds to sleep (0-999999)
-   * @throws InterruptedException if the Thread is interrupted; it's
-   *         <i>interrupted status</i> will be cleared
-   * @throws IllegalArgumentException if ns is invalid
-   * @see #notify()
-   * @see #wait(long, int)
+   * @throws InterruptedException if the Thread is (or was) interrupted;
+   *         it's <i>interrupted status</i> will be cleared
+   * @throws IllegalArgumentException if ms or ns is negative
+   *         or ns is larger than 999999.
+   * @see #interrupt()
    */
   public static void sleep(long ms, int ns) throws InterruptedException
   {
-    if(ms < 0 || ns < 0 || ns > 999999)
-	throw new IllegalArgumentException();
 
+    // Check parameters
+    if (ms < 0 || ns < 0 || ns > 999999)
+      throw new IllegalArgumentException();
+
+    // Really sleep
     VMThread.sleep(ms, ns);
   }
 
@@ -955,9 +963,7 @@ public class Thread implements Runnable
   public final synchronized void suspend()
   {
     checkAccess();
-    VMThread t = vmThread;
-    if (t != null)
-	t.suspend();
+    VMThread.suspend(vmData);
   }
 
   /**
@@ -1008,5 +1014,20 @@ public class Thread implements Runnable
   {
     group.removeThread(this);
     vmThread = null;
+    locals = null;
+  }
+
+  /**
+   * Returns the map used by ThreadLocal to store the thread local values.
+   */
+  static Map getThreadLocals()
+  {
+    Thread thread = currentThread();
+    Map locals = thread.locals;
+    if (locals == null)
+      {
+        locals = thread.locals = new WeakHashMap();
+      }
+    return locals;
   }
 }
