@@ -39,6 +39,7 @@ exception statement from your version. */
 package java.util;
 
 import gnu.classpath.SystemProperties;
+import gnu.java.locale.LocaleHelper;
 
 import java.io.IOException;
 import java.io.ObjectInputStream;
@@ -78,6 +79,7 @@ import java.io.Serializable;
  * @author Jochen Hoenicke
  * @author Paul Fisher
  * @author Eric Blake (ebb9@email.byu.edu)
+ * @author Andrew John Hughes (gnu_andrew@member.fsf.org)
  * @since 1.1
  * @status updated to 1.4
  */
@@ -194,14 +196,14 @@ public final class Locale implements Serializable, Cloneable
   /**
    * Array storing all available locales.
    */
-  private transient static Locale[] availableLocales;
+  private static transient Locale[] availableLocales;
 
   /**
    * Locale cache. Only created locale objects are stored.
    * Contains all supported locales when getAvailableLocales()
    * got called.
    */
-  private transient static HashMap localeMap;
+  private static transient HashMap localeMap;
   
   /**
    * The default locale. Except for during bootstrapping, this should never be
@@ -210,19 +212,52 @@ public final class Locale implements Serializable, Cloneable
    */
   private static Locale defaultLocale =
     getLocale(SystemProperties.getProperty("user.language", "en"),
-              SystemProperties.getProperty("user.region", ""),
+              SystemProperties.getProperty("user.region", "US"),
               SystemProperties.getProperty("user.variant", ""));
- 
+
+  /**
+   * Array storing all the available two-letter ISO639 languages.
+   */
+  private static transient String[] languageCache;
+
+  /**
+   * Array storing all the available two-letter ISO3166 country codes.
+   */
+  private static transient String[] countryCache;
+
+  /**
+   * Retrieves the locale with the specified language from the cache.
+   *
+   * @param language the language of the locale to retrieve.
+   * @return the locale.
+   */ 
   private static Locale getLocale(String language)
   {
     return getLocale(language, "", "");
   }
   
+  /**
+   * Retrieves the locale with the specified language and region
+   * from the cache.
+   *
+   * @param language the language of the locale to retrieve.
+   * @param region the region of the locale to retrieve.
+   * @return the locale.
+   */ 
   private static Locale getLocale(String language, String region)
   {
     return getLocale(language, region, "");
   }
   
+  /**
+   * Retrieves the locale with the specified language, region
+   * and variant from the cache.
+   *
+   * @param language the language of the locale to retrieve.
+   * @param region the region of the locale to retrieve.
+   * @param variant the variant of the locale to retrieve.
+   * @return the locale.
+   */ 
   private static Locale getLocale(String language, String region, String variant)
   {
     if (localeMap == null)
@@ -346,7 +381,7 @@ public final class Locale implements Serializable, Cloneable
    *
    * @return the installed locales
    */
-  public synchronized static Locale[] getAvailableLocales()
+  public static synchronized Locale[] getAvailableLocales()
   {
     if (availableLocales == null)
       {
@@ -387,29 +422,12 @@ public final class Locale implements Serializable, Cloneable
    */
   public static String[] getISOCountries()
   {
-    return new String[]
-    {
-      "AD", "AE", "AF", "AG", "AI", "AL", "AM", "AN", "AO", "AQ", "AR", "AS",
-      "AT", "AU", "AW", "AZ", "BA", "BB", "BD", "BE", "BF", "BG", "BH", "BI",
-      "BJ", "BM", "BN", "BO", "BR", "BS", "BT", "BV", "BW", "BY", "BZ", "CA",
-      "CC", "CF", "CG", "CH", "CI", "CK", "CL", "CM", "CN", "CO", "CR", "CU",
-      "CV", "CX", "CY", "CZ", "DE", "DJ", "DK", "DM", "DO", "DZ", "EC", "EE",
-      "EG", "EH", "ER", "ES", "ET", "FI", "FJ", "FK", "FM", "FO", "FR", "FX",
-      "GA", "GB", "GD", "GE", "GF", "GH", "GI", "GL", "GM", "GN", "GP", "GQ",
-      "GR", "GS", "GT", "GU", "GW", "GY", "HK", "HM", "HN", "HR", "HT", "HU",
-      "ID", "IE", "IL", "IN", "IO", "IQ", "IR", "IS", "IT", "JM", "JO", "JP",
-      "KE", "KG", "KH", "KI", "KM", "KN", "KP", "KR", "KW", "KY", "KZ", "LA",
-      "LB", "LC", "LI", "LK", "LR", "LS", "LT", "LU", "LV", "LY", "MA", "MC",
-      "MD", "MG", "MH", "MK", "ML", "MM", "MN", "MO", "MP", "MQ", "MR", "MS",
-      "MT", "MU", "MV", "MW", "MX", "MY", "MZ", "NA", "NC", "NE", "NF", "NG",
-      "NI", "NL", "NO", "NP", "NR", "NU", "NZ", "OM", "PA", "PE", "PF", "PG",
-      "PH", "PK", "PL", "PM", "PN", "PR", "PT", "PW", "PY", "QA", "RE", "RO",
-      "RU", "RW", "SA", "SB", "SC", "SD", "SE", "SG", "SH", "SI", "SJ", "SK",
-      "SL", "SM", "SN", "SO", "SR", "ST", "SV", "SY", "SZ", "TC", "TD", "TF",
-      "TG", "TH", "TJ", "TK", "TM", "TN", "TO", "TP", "TR", "TT", "TV", "TW",
-      "TZ", "UA", "UG", "UM", "US", "UY", "UZ", "VA", "VC", "VE", "VG", "VI",
-      "VN", "VU", "WF", "WS", "YE", "YT", "YU", "ZA", "ZM", "ZR", "ZW"
-    };
+    if (countryCache == null)
+      {
+	countryCache = getISOStrings("territories");
+      }
+
+    return countryCache;
   }
 
   /**
@@ -420,21 +438,50 @@ public final class Locale implements Serializable, Cloneable
    */
   public static String[] getISOLanguages()
   {
-    return new String[]
-    {
-      "aa", "ab", "af", "am", "ar", "as", "ay", "az", "ba", "be", "bg", "bh",
-      "bi", "bn", "bo", "br", "ca", "co", "cs", "cy", "da", "de", "dz", "el",
-      "en", "eo", "es", "et", "eu", "fa", "fi", "fj", "fo", "fr", "fy", "ga",
-      "gd", "gl", "gn", "gu", "ha", "he", "hi", "hr", "hu", "hy", "ia", "id",
-      "ie", "ik", "in", "is", "it", "iu", "iw", "ja", "ji", "jw", "ka", "kk",
-      "kl", "km", "kn", "ko", "ks", "ku", "ky", "la", "ln", "lo", "lt", "lv",
-      "mg", "mi", "mk", "ml", "mn", "mo", "mr", "ms", "mt", "my", "na", "ne",
-      "nl", "no", "oc", "om", "or", "pa", "pl", "ps", "pt", "qu", "rm", "rn",
-      "ro", "ru", "rw", "sa", "sd", "sg", "sh", "si", "sk", "sl", "sm", "sn",
-      "so", "sq", "sr", "ss", "st", "su", "sv", "sw", "ta", "te", "tg", "th",
-      "ti", "tk", "tl", "tn", "to", "tr", "ts", "tt", "tw", "ug", "uk", "ur",
-      "uz", "vi", "vo", "wo", "xh", "yi", "yo", "za", "zh", "zu"
-    };
+    if (languageCache == null)
+      {
+	languageCache = getISOStrings("languages");
+      }
+    return languageCache;
+  }
+
+  /**
+   * Returns the set of keys from the specified resource hashtable, filtered
+   * so that only two letter strings are returned.
+   *
+   * @param tableName the name of the table from which to retrieve the keys.
+   * @return an array of two-letter strings.
+   */
+  private static String[] getISOStrings(String tableName)
+  {
+    List tempList;
+    ResourceBundle bundle;
+    Enumeration keys;
+    int count = 0;
+    String[] strings;
+
+    tempList = new ArrayList();
+    bundle = ResourceBundle.getBundle("gnu.java.locale.LocaleInformation");
+    keys = ((Hashtable) bundle.getObject(tableName)).keys();
+    while (keys.hasMoreElements())
+      {
+	String nextString;
+
+	nextString = (String) keys.nextElement();
+	if (nextString.length() == 2 &&
+	    Character.isLetter(nextString.charAt(0)) &&
+	    Character.isLetter(nextString.charAt(1)))
+	  {
+	    tempList.add(nextString);
+	    ++count;
+	  }
+      }
+    strings = new String[count];
+    for (int a = 0; a < count; ++a)
+      {
+	strings[a] = (String) tempList.get(a);
+      }
+    return strings;
   }
 
   /**
@@ -590,25 +637,34 @@ public final class Locale implements Serializable, Cloneable
   }
 
   /**
-   * Gets the language name suitable for display to the user, formatted
-   * for a specified locale.
+   * <p>
+   * Gets the name of the language specified by this locale, in a form suitable
+   * for display to the user.  If possible, the display name will be localized
+   * to the specified locale.  For example, if the locale instance is
+   * <code>Locale.GERMANY</code>, and the specified locale is <code>Locale.UK</code>,
+   * the result would be 'German'.  Using the German locale would instead give
+   * 'Deutsch'.  If the display name can not be localized to the supplied
+   * locale, it will fall back on other output in the following order:
+   * </p>
+   * <ul>
+   * <li>the display name in the default locale</li>
+   * <li>the display name in English</li>
+   * <li>the ISO code</li>
+   * </ul>
+   * <p>
+   * If the language is unspecified by this locale, then the empty string is
+   * returned.
+   * </p>
    *
-   * @param locale locale to use for formatting
+   * @param inLocale the locale to use for formatting the display string.
    * @return the language name of this locale localized to the given locale,
-   *         with the ISO code as backup
+   *         with the default locale, English and the ISO code as backups.
+   * @throws NullPointerException if the supplied locale is null.
    */
-  public String getDisplayLanguage(Locale locale)
+  public String getDisplayLanguage(Locale inLocale)
   {
-    try
-      {
-        ResourceBundle bundle
-          = ResourceBundle.getBundle("java.util.iso639", locale);
-        return bundle.getString(language);
-      }
-    catch (MissingResourceException ex)
-      {
-        return language;
-      }
+    return LocaleHelper.getLocalizedString(inLocale, language,
+					   "languages", true, false);
   }
 
   /**
@@ -628,25 +684,34 @@ public final class Locale implements Serializable, Cloneable
   }
 
   /**
-   * Gets the country name suitable for display to the user, formatted
-   * for a specified locale.
+   * <p>
+   * Gets the name of the country specified by this locale, in a form suitable
+   * for display to the user.  If possible, the display name will be localized
+   * to the specified locale.  For example, if the locale instance is
+   * <code>Locale.GERMANY</code>, and the specified locale is <code>Locale.UK</code>,
+   * the result would be 'Germany'.  Using the German locale would instead give
+   * 'Deutschland'.  If the display name can not be localized to the supplied
+   * locale, it will fall back on other output in the following order:
+   * </p>
+   * <ul>
+   * <li>the display name in the default locale</li>
+   * <li>the display name in English</li>
+   * <li>the ISO code</li>
+   * </ul>
+   * <p>
+   * If the country is unspecified by this locale, then the empty string is
+   * returned.
+   * </p>
    *
-   * @param locale locale to use for formatting
+   * @param inLocale the locale to use for formatting the display string.
    * @return the country name of this locale localized to the given locale,
-   *         with the ISO code as backup
+   *         with the default locale, English and the ISO code as backups.
+   * @throws NullPointerException if the supplied locale is null.
    */
-  public String getDisplayCountry(Locale locale)
+  public String getDisplayCountry(Locale inLocale)
   {
-    try
-      {
-        ResourceBundle bundle =
-          ResourceBundle.getBundle("java.util.iso3166", locale);
-        return bundle.getString(country);
-      }
-    catch (MissingResourceException ex)
-      {
-        return country;
-      }
+    return LocaleHelper.getLocalizedString(inLocale, country,
+					    "territories", true, false);
   }
 
   /**
@@ -665,19 +730,36 @@ public final class Locale implements Serializable, Cloneable
     return getDisplayVariant(defaultLocale);
   }
 
+
   /**
-   * Returns the variant name of this locale localized to the
-   * given locale. If the localized is not found, the variant code
-   * itself is returned.
+   * <p>
+   * Gets the name of the variant specified by this locale, in a form suitable
+   * for display to the user.  If possible, the display name will be localized
+   * to the specified locale.  For example, if the locale instance is a revised
+   * variant, and the specified locale is <code>Locale.UK</code>, the result
+   * would be 'REVISED'.  Using the German locale would instead give
+   * 'Revidiert'.  If the display name can not be localized to the supplied
+   * locale, it will fall back on other output in the following order:
+   * </p>
+   * <ul>
+   * <li>the display name in the default locale</li>
+   * <li>the display name in English</li>
+   * <li>the ISO code</li>
+   * </ul>
+   * <p>
+   * If the variant is unspecified by this locale, then the empty string is
+   * returned.
+   * </p>
    *
-   * @param locale locale to use for formatting
-   * @return the variant code of this locale localized to the given locale,
-   *         with the ISO code as backup
+   * @param inLocale the locale to use for formatting the display string.
+   * @return the variant name of this locale localized to the given locale,
+   *         with the default locale, English and the ISO code as backups.
+   * @throws NullPointerException if the supplied locale is null.
    */
-  public String getDisplayVariant(Locale locale)
+  public String getDisplayVariant(Locale inLocale)
   {
-    // XXX - load a bundle?
-    return variant;
+    return LocaleHelper.getLocalizedString(inLocale, variant, "variants",
+					   true, false);
   }
 
   /**
