@@ -662,7 +662,7 @@ public abstract class AbstractButton extends JComponent
    *
    * @param e The event signifying that the button's model changed state
    */
-  public void fireItemStateChanged(ItemEvent e)
+  protected void fireItemStateChanged(ItemEvent e)
   {
     e.setSource(this);
     ItemListener[] listeners = getItemListeners();
@@ -677,7 +677,7 @@ public abstract class AbstractButton extends JComponent
    *
    * @param e The event signifying that the button's model was clicked
    */
-  public void fireActionPerformed(ActionEvent e)
+  protected void fireActionPerformed(ActionEvent e)
   {
     ActionListener[] listeners = getActionListeners();
     
@@ -700,7 +700,7 @@ public abstract class AbstractButton extends JComponent
    * Calls {@link ChangeEvent.stateChanged} on each {@link ChangeListener}
    * in the button's listener list.
    */
-  public void fireStateChanged()
+  protected void fireStateChanged()
   {
     ChangeListener[] listeners = getChangeListeners();
 
@@ -817,7 +817,7 @@ public abstract class AbstractButton extends JComponent
   public void setRolloverEnabled(boolean r)
   {
     if (rollOverEnabled != r)
-  {
+      {
         rollOverEnabled = r;
         firePropertyChange(ROLLOVER_ENABLED_CHANGED_PROPERTY, !r, r);
         revalidate();
@@ -1520,7 +1520,7 @@ public abstract class AbstractButton extends JComponent
         public void propertyChange(PropertyChangeEvent e)
         {
           Action act = (Action) (e.getSource());	
-	  if (e.getPropertyName().equals(AbstractAction.ENABLED_PROPERTY))
+	  if (e.getPropertyName().equals("enabled"))
 	    setEnabled(act.isEnabled());
 	  else if (e.getPropertyName().equals(Action.NAME))
             setText((String)(act.getValue(Action.NAME)));
@@ -1858,7 +1858,39 @@ public abstract class AbstractButton extends JComponent
    */
   protected String paramString()
   {
-    return "AbstractButton";
+    StringBuffer sb = new StringBuffer();
+    sb.append(super.paramString());
+    sb.append(",defaultIcon=");
+    if (getIcon() != null)
+      sb.append(getIcon());
+    sb.append(",disabledIcon=");
+    if (getDisabledIcon() != null)
+      sb.append(getDisabledIcon());
+    sb.append(",disabledSelectedIcon=");
+    if (getDisabledSelectedIcon() != null)
+      sb.append(getDisabledSelectedIcon());
+    sb.append(",margin=");
+    if (getMargin() != null)
+      sb.append(getMargin());
+    sb.append(",paintBorder=").append(isBorderPainted());
+    sb.append(",paintFocus=").append(isFocusPainted());
+    sb.append(",pressedIcon=");
+    if (getPressedIcon() != null)
+      sb.append(getPressedIcon());
+    sb.append(",rolloverEnabled=").append(isRolloverEnabled());
+    sb.append(",rolloverIcon=");
+    if (getRolloverIcon() != null)
+      sb.append(getRolloverIcon());
+    sb.append(",rolloverSelected=");
+    if (getRolloverSelectedIcon() != null)
+      sb.append(getRolloverSelectedIcon());
+    sb.append(",selectedIcon=");
+    if (getSelectedIcon() != null)
+      sb.append(getSelectedIcon());
+    sb.append(",text=");
+    if (getText() != null)
+      sb.append(getText());
+    return sb.toString();
   }
 
   /**
