@@ -3,7 +3,7 @@
 
 Modifications Copyright (C) 2004 by Etienne Gagnon.
 Modifications Copyright (C) 2004 by David Belanger.
-Modifications Copyright (C) 2004 by Grzegorz prokopski.
+Modifications Copyright (C) 2004, 2005 by Grzegorz Prokopski.
 
 This file is part of GNU Classpath.
 
@@ -58,9 +58,9 @@ Systems    : all
 
 #include <stdlib.h>
 
-#ifdef __APPLE__
+#ifndef HAVE_SOCKLEN_T
 typedef unsigned int socklen_t;
-#endif /* __APPLE__ */
+#endif /* HAVE_SOCKLEN_T */
 
 #include "target_native.h"
 
