@@ -1,10 +1,10 @@
 /* VMClassLoader.java -- Reference implementation of native interface
    required by ClassLoader
-   Copyright (C) 1998, 2001, 2002, 2004 Free Software Foundation
+   Copyright (C) 1998, 2001, 2002, 2004, 2005 Free Software Foundation
 
 Modifications Copyright (C) 2004 by Etienne Gagnon.
 Modifications Copyright (C) 2004 by David Belanger.
-Modifications Copyright (C) 2004 by Grzegorz prokopski.
+Modifications Copyright (C) 2004, 2005 by Grzegorz Prokopski.
 
 This file is part of GNU Classpath.
 
@@ -44,20 +44,18 @@ exception statement from your version. */
 package java.lang;
 
 import gnu.classpath.SystemProperties;
-import gnu.java.lang.SystemClassLoader;
-import gnu.java.util.EmptyEnumeration;
 
 import java.io.File;
 import java.io.IOException;
-import java.lang.reflect.Constructor;
 import java.net.MalformedURLException;
 import java.net.URL;
 import java.security.ProtectionDomain;
 import java.util.Enumeration;
-import java.util.Map;
 import java.util.HashMap;
+import java.util.Map;
 import java.util.StringTokenizer;
 import java.util.Vector;
+import java.util.zip.ZipFile;
 
 /**
  * java.lang.VMClassLoader is a package-private helper for VMs to implement
@@ -235,16 +233,55 @@ final class VMClassLoader
     Vector v = new Vector();
     while (st.hasMoreTokens())
       {
-	File file = new File(st.nextToken(), name);
-	if (!file.exists())
-	  continue;
-	try
+	File file = new File(st.nextToken());
+	if (file.isDirectory())
 	  {
-	    v.add(new URL("file://" + file.getAbsolutePath()));
+	    try
+	      {
+		v.add(new URL("file://"
+		  + new File(file, name).getAbsolutePath()));
+	      }
+	    catch (MalformedURLException e)
+	      {
+		throw new Error(e);
+	      }
 	  }
-	catch (MalformedURLException e)
+	else if (file.isFile())
 	  {
-	    throw new Error(e);
+	    ZipFile zip;
+	    try
+	      {
+		zip = new ZipFile(file);
+	      }
+	    catch (IOException e)
+	      {
+		continue;
+	      }
+	    String zname = name.startsWith("/") ? name.substring(1) : name;
+	    try
+	      {
+		if (zip.getEntry(zname) == null)
+		    continue;
+	      }
+	    finally
+	      {
+		try
+		  {
+		    zip.close();
+		  }
+		catch (IOException e)
+		  {
+		  }
+	      }
+	    try
+	      {
+		v.add(new URL("jar:file://"
+		  + file.getAbsolutePath() + "!/" + zname));
+	      }
+	    catch (MalformedURLException e)
+	      {
+		throw new Error(e);
+	      }
 	  }
       }
     return v.elements();
