@@ -3,7 +3,7 @@
 
 Modifications Copyright (C) 2004 by Etienne Gagnon.
 Modifications Copyright (C) 2004 by David Belanger.
-Modifications Copyright (C) 2004 by Grzegorz prokopski.
+Modifications Copyright (C) 2004, 2005 by Grzegorz Prokopski.
 
 This file is part of GNU Classpath.
 
@@ -43,7 +43,6 @@ package java.lang;
 
 import java.io.File;
 import java.io.IOException;
-import java.util.Properties;
 
 /**
  * VMRuntime represents the interface to the Virtual Machine.
@@ -155,9 +154,10 @@ final class VMRuntime
      * already been mapped to a true filename.
      *
      * @param filename the file to load
+     * @param loader class loader, or <code>null</code> for the boot loader
      * @return 0 on failure, nonzero on success
      */
-    static native int nativeLoad(String filename);
+    static native int nativeLoad(String filename, ClassLoader loader);
 
     /**
      * Map a system-independent "short name" to the full file name.
