@@ -1,9 +1,9 @@
 /* VMThread -- VM interface for Thread of executable code
-   Copyright (C) 2003, 2004 Free Software Foundation
+   Copyright (C) 2003, 2004, 2005 Free Software Foundation
 
 Modifications Copyright (C) 2004 by Etienne Gagnon.
 Modifications Copyright (C) 2004 by David Belanger.
-Modifications Copyright (C) 2004 by Grzegorz prokopski.
+Modifications Copyright (C) 2004, 2005 by Grzegorz Prokopski.
 
 This file is part of GNU Classpath.
 
@@ -60,7 +60,6 @@ package java.lang;
  * <li>native void nativeStop(Throwable t);
  * <li>native static Thread currentThread();
  * <li>static native void yield();
- * <li>static native void sleep(long ms, int ns) throws InterruptedException;
  * <li>static native boolean interrupted();
  * </ul>
  * All other methods may be implemented to make Thread handling more efficient
@@ -91,7 +90,7 @@ final class VMThread
      *
      * @param thread The Thread object that was just created.
      */
-    private VMThread(Thread thread)
+    /* private */ VMThread(Thread thread)
     {
 	this.thread = thread;
     }
@@ -244,14 +243,17 @@ final class VMThread
      */
     synchronized void join(long ms, int ns) throws InterruptedException
     {
- 	// round up
+	// Round up
 	ms += (ns != 0) ? 1 : 0;
 
-	long end = System.currentTimeMillis() + ms;
+	// Compute end time, but don't overflow
+	long now = System.currentTimeMillis();
+	long end = now + ms;
+	if (end < now)
+	    end = Long.MAX_VALUE;
 
-	// Apparently, some VMs will return from wait without notify having
-	// been called, so we loop and test the vmThread field in our
-	// corresponding Thread object.
+	// A VM is allowed to return from wait() without notify() having been
+	// called, so we loop to handle possible spurious wakeups.
 	while(thread.vmThread != null)
 	{
 	    // We use the VMThread object to wait on, because this is a private
@@ -259,7 +261,7 @@ final class VMThread
 	    wait(ms);
 	    if(ms != 0)
 	    {
-		long now = System.currentTimeMillis();
+		now = System.currentTimeMillis();
 		ms = end - now;
 		if(ms <= 0)
 		{
@@ -365,13 +367,13 @@ final class VMThread
     /**
      * Suspend this Thread.  It will not come back, ever, unless it is resumed.
      */
-    native void suspend();
+    native static void suspend(byte[] vmData);
 
     /**
      * Resume this Thread.  If the thread is not suspended, this method does
      * nothing.
      */
-    native void resume();
+    native static void resume(byte[] vmData);
 
     /**
      * Set the priority of the underlying platform thread.
@@ -395,7 +397,7 @@ final class VMThread
      *
      * @return the currently executing Thread
      */
-    native static Thread currentThread();
+    static native Thread currentThread();
 
     /**
      * Yield to another thread. The Thread will not lose any locks it holds
@@ -417,26 +419,52 @@ final class VMThread
      * because some other thread may be active.  So don't expect real-time
      * performance.
      *
-     * @param ms the number of milliseconds to sleep
+     * @param ms the number of milliseconds to sleep.
      * @param ns the number of extra nanoseconds to sleep (0-999999)
-     * @throws InterruptedException if the Thread is interrupted; it's
-     *         <i>interrupted status</i> will be cleared
-     * @throws IllegalArgumentException if ns is invalid
+     * @throws InterruptedException if the Thread is (or was) interrupted;
+     *         it's <i>interrupted status</i> will be cleared
      */
-  //static native void sleep(long ms, int ns) throws InterruptedException;
-  public static void sleep(long ms, int ns) throws InterruptedException
-  {
-    Thread thread = currentThread();
-    synchronized(thread.sleeper)
-      {
-	if(ms != 0 || ns != 0)
-          thread.sleeper.wait(ms, ns);
-	else
-	  thread.yield(); // trying to be a good citizen
-      }
-  }
-    
+    static native void sleep(long ms, int ns) throws InterruptedException;
 
+/*
+    static void sleep(long ms, int ns) throws InterruptedException
+    {    
+      // Round up
+      ms += (ns != 0) ? 1 : 0;
+
+      // Note: JDK treats a zero length sleep is like Thread.yield(),
+      // without checking the interrupted status of the thread.
+      // It's unclear if this is a bug in the implementation or the spec.
+      // See http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=6213203
+      if (ms == 0)
+	{
+	  if (Thread.interrupted())
+	    throw new InterruptedException();
+	  return;
+	}
+
+      // Compute end time, but don't overflow
+      long now = System.currentTimeMillis();
+      long end = now + ms;
+      if (end < now)
+	  end = Long.MAX_VALUE;
+
+      // A VM is allowed to return from wait() without notify() having been
+      // called, so we loop to handle possible spurious wakeups.
+      VMThread vt = Thread.currentThread().vmThread;
+      synchronized (vt)
+	{
+	  while (true)
+	    {
+	      vt.wait(ms);
+	      now = System.currentTimeMillis();
+	      if (now >= end)
+		break;
+	      ms = end - now;
+	    }
+	}
+    }
+*/
 
     /**
      * Determine whether the current Thread has been interrupted, and clear
