@@ -3,7 +3,7 @@
  
 Modifications Copyright (C) 2004 by Etienne Gagnon.
 Modifications Copyright (C) 2004 by David Belanger.
-Modifications Copyright (C) 2004 by Grzegorz prokopski.
+Modifications Copyright (C) 2004, 2005 by Grzegorz Prokopski.
 
 This file is part of the SableVM class library.
  
@@ -41,6 +41,7 @@ exception statement from your version. */
 
 package java.lang;
 
+import java.io.*;
 import java.util.*;
 import java.lang.reflect.*;
 
@@ -76,6 +77,21 @@ class VirtualMachine
     e.printStackTrace();
   }
 
+  private static String exceptionDescription(Throwable e) throws IOException
+  {
+      StringWriter sw = new StringWriter();
+      PrintWriter pw = new PrintWriter(sw);
+      e.printStackTrace(pw);
+      pw.close();
+      sw.close();
+      return sw.toString();
+  }
+
+  private static String exceptionTruncatedDescription(Throwable e)
+  {
+      return e.toString();
+  }
+
   private static void main(String[] args)
   throws ClassNotFoundException
   {
