@@ -2,8 +2,8 @@
    Copyright (C) 1998, 2001, 2002 Free Software Foundation, Inc.
 
 Modifications Copyright (C) 2004 by Etienne Gagnon.
-Modifications Copyright (C) 2004 by David Belanger.
-Modifications Copyright (C) 2004 by Grzegorz prokopski.
+Modifications Copyright (C) 2004, 2005 by David Belanger.
+Modifications Copyright (C) 2004, 2005 by Grzegorz Prokopski.
 
 This file is part of GNU Classpath.
 
@@ -172,7 +172,9 @@ extends AccessibleObject implements Member
 
     if (returnType == null)
     {
-      returnType = ReflectUtil.getReturnType(nativeGetDescriptor(vmData));
+      returnType =
+	ReflectUtil.getReturnType(nativeGetDescriptor(vmData),
+				  getDeclaringClass().getClassLoader());
     }
     return returnType;
   }
@@ -189,7 +191,8 @@ extends AccessibleObject implements Member
     if (parameterTypes == null)
     {
       parameterTypes =
-        ReflectUtil.getParameterTypes(nativeGetDescriptor(vmData));
+        ReflectUtil.getParameterTypes(nativeGetDescriptor(vmData),
+				      getDeclaringClass().getClassLoader());
     }
     return parameterTypes;
   }
@@ -404,10 +407,6 @@ extends AccessibleObject implements Member
   public Object invoke(Object o, Object[] args)
   throws IllegalAccessException, InvocationTargetException
   {
-    /* The following code is more of a hack, than real
-     * implementation, as it lacks checking and widening. To be
-     * fixed...*/
-
     if (args == null) {
       args = new Object[0];
     } else {
@@ -431,9 +430,7 @@ extends AccessibleObject implements Member
       if (args.getClass() != Object[].class) {
 	Object[] newArgs;
 	newArgs = new Object[args.length];
-	for (int i = 0; i < args.length; i++) {
-	  newArgs[i] = args[i];
-	}
+	System.arraycopy(args, 0, newArgs, 0, args.length);
 	// ugly, changing param value
 	args = newArgs;
       }
@@ -441,11 +438,17 @@ extends AccessibleObject implements Member
     }
 
     char[] paramTypes = getParamTypes();
+    Class[] parameterTypes = getParameterTypes();
     int count = paramTypes.length;
 
+    if (count != args.length)
+      {
+	throw new IllegalArgumentException("incorrect number of arguments");
+      }
+
+    try {
     for (int i = 0; i < count; i++)
     {
-      /* In the future, we should handle primitive type widening. */
       switch (paramTypes[i])
       {
       case 'Z':
@@ -515,7 +518,12 @@ extends AccessibleObject implements Member
 
       case 'L':
         {
-          /* should be checking type */
+	  // check type
+	  if (args[i] != null) {
+	    if (!(parameterTypes[i].isAssignableFrom(args[i].getClass()))) {
+	      throw new IllegalArgumentException("argument of incorrect type");
+	    }
+	  }
         }
         break;
 
@@ -523,6 +531,9 @@ extends AccessibleObject implements Member
         throw new InternalError();
       }
     }
+    } catch (ClassCastException e) {
+      throw new IllegalArgumentException("argument of incorrect type");	
+    }
 
     Object result;
 
